#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file tk_stp_state_viewer.py
 @brief This module provides the graphical user interface
 @date $Date$

 @author 株式会社 東日本計算センター

 NewBSD

"""
import os.path
import sys
import time
# import RTC
# import STPPortIF
import Tkinter as Tk
import ttk
# import random
# import copy
from PIL import Image, ImageTk
from logging import getLogger, StreamHandler
from logging import INFO as LOG_LV

sys.path.append(".")

# Setup logger
logger = getLogger(__name__)
handler = StreamHandler()
logger.setLevel(LOG_LV)
handler.setLevel(LOG_LV)
logger.addHandler(handler)

# Set constants
NODE = 1
ROUTE = 2
INFO = 4
ALL = NODE | ROUTE | INFO

# CURRENT = os.getcwd()
# IMG_PATH = CURRENT + "/STP_network_topology.png"
from stp_nw_image_generator import IMG_PATH


class NodeList(Tk.Frame):

    def __init__(self, master=None):
        Tk.Frame.__init__(self, master)

        self._init_variable()
        self._setup_frame()

    def _init_variable(self):
        self._tree = None
        self._sel_id = None
        self._plugin_func = None
        self._is_descending = False

    def _setup_frame(self):
        # configure node list
        self._tree = ttk.Treeview(self, columns="Node",
                                  show="headings", selectmode="browse")

        cb_func = (lambda col="Node":
                   self._cb_sort_order_change(col, not self._is_descending))

        self._tree.heading("Node", text="Node",
                           anchor=Tk.CENTER, command=cb_func)

        self._tree.column("Node", stretch=True, anchor=Tk.CENTER,
                          minwidth=180, width=180)
        self._tree.bind("<<TreeviewSelect>>", self._cb_sel_node_change)

        # configure scrollbar
        sb = ttk.Scrollbar(self, orient=Tk.VERTICAL, command=self._tree.yview)
        self._tree.configure(yscroll=sb.set)

        # add configured widgets
        self._tree.grid(row=0, column=0, sticky=Tk.NSEW)
        sb.grid(row=0, column=1, sticky=Tk.NS)

        # set resize properties
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)

    def add(self, node_id):
        self._tree.insert("", "end", value=node_id, tags=node_id)
        self._tree.tag_configure(node_id, foreground="black")

        self._cb_sort_order_change("Node", self._is_descending)

    def clear(self):
        node_list = self._tree.get_children()
        for i in node_list:
            self._tree.delete(i)

        self._selected_node = None

    def set_missing_node(self, node_id, is_missing):
        child = self._tree.get_children()

        target = None
        for item in child:
            if node_id == str(self._tree.item(item)['values'][0]):
                target = item
                break

        if target:
            color = "black"
            if is_missing:
                color = "red"

            self._tree.tag_configure(node_id, foreground=color)

    def get_node_count(self):
        cnt = len(self._tree.get_children())
        return cnt

    def get_sel_node_id(self):
        return self._sel_id

    def set_sel_node_id(self, node_id):
        child = self._tree.get_children()

        target = None
        for item in child:
            if node_id == str(self._tree.item(item)['values'][0]):
                target = item
                break

        if target:
            self._tree.selection_set(target)

    def plugin_sel_node_change(self, func):
        self._plugin_func = func

    def _cb_sel_node_change(self, event):
        sel_item = self._tree.selection()

        if not sel_item:
            return

        child = self._tree.get_children()

        for item in child:
            if sel_item[0] == item:
                self._sel_id = str(self._tree.item(item)['values'][0])
                break

        if self._plugin_func:
            self._plugin_func()  # TkSTPStateViewer._cb_selected_node_change

    def _cb_sort_order_change(self, col, sort_order):
        child = [(self._tree.set(child, col), child)
                 for child in self._tree.get_children()]

        child.sort(reverse=sort_order)

        for i, item in enumerate(child):
            self._tree.move(item[1], "", i)

        self._is_descending = sort_order


class RouteList(Tk.Frame):

    def __init__(self, master=None):
        Tk.Frame.__init__(self, master)

        self._init_variable()
        self._setup_frame()

    def _init_variable(self):
        self._tree = None
        self._is_descending = False

    def _setup_frame(self):
        # configure route list
        col_name = ("Interface", "Connected Node",
                    "Connected Port", "Port State")
        self._tree = ttk.Treeview(self, columns=col_name, show="headings")

        for name in col_name:
            cb_func = (lambda col=name:
                       self._cb_sort_order_change(col,
                                                  not self._is_descending))

            self._tree.heading(name, text=name.title(),
                               anchor=Tk.CENTER,
                               command=cb_func)

            self._tree.column(name, stretch=True, anchor=Tk.CENTER,
                              minwidth=150, width=150)

        # configure scrollbar
        sb = ttk.Scrollbar(self, orient=Tk.VERTICAL, command=self._tree.yview)
        self._tree.configure(yscroll=sb.set)

        # add configured widgets
        self._tree.grid(row=0, column=0, sticky=Tk.NSEW)
        sb.grid(row=0, column=1, sticky=Tk.NS)

        # set resize properties
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)

    def add(self, dest, mask, gway, iface, tagname=None):
        self._tree.insert("", "end",
                          values=(dest, mask, gway, iface),
                          tags="route")

    def clear(self):
        child = self._tree.get_children()
        for i in child:
            self._tree.delete(i)

    def show_entire_route(self, route):
        self.clear()

        if route:
            for i in route:
                self.add(i.NodeInterface, i.Connected_Node,
                         i.Connected_Port, i.PortState,)

            self._tree.tag_configure("route", foreground="black")
        else:
            self.add("no data", "no data", "no data", "no data")
            self._tree.tag_configure("route", foreground="red")

    def _cb_sort_order_change(self, col, sort_order):
        child = [(self._tree.set(child, col), child)
                 for child in self._tree.get_children()]

        child.sort(reverse=sort_order)

        for i, item in enumerate(child):
            self._tree.move(item[1], "", i)

        self._is_descending = sort_order


class InformationBar(Tk.Frame):

    def __init__(self, master=None):
        Tk.Frame.__init__(self, master)

        self._init_variable()
        self._setup_frame()

    def _init_variable(self):
        self._label = None

    def _setup_frame(self):
        self._label = Tk.Label(self, bd=1, relief=Tk.SUNKEN, anchor=Tk.E)
        self._label.pack(fill=Tk.X)

    def set(self, format, *args):
        color = "black"

        if not args:
            color = "red"

        self._label.config(text=format % args, fg=color)
        self._label.update_idletasks()

    def clear(self):
        self._label.config(text="")
        self._label.update_idletasks()


# h28-n
# NetworkTopology
class NetworkTopology(Tk.Frame):

    def __init__(self, master=None):
        Tk.Frame.__init__(self, master)

        self._init_variable()
        self._setup_frame()

    def _init_variable(self):
        self._label = None

    def _setup_frame(self):
        self._label = Tk.Label(self, bd=1, relief=Tk.SUNKEN, anchor=Tk.CENTER)
        self._label.pack(fill=Tk.X)

    def show_nwtopology(self):
        self._image_del()
        self._image_open()

    def _image_open(self):
        if os.path.isfile(IMG_PATH):
            readimage = Image.open(IMG_PATH, 'r')
            self.img = ImageTk.PhotoImage(readimage)
            self._label.configure(image=self.img)
            self._label.pack()
        else:
            logger.error("invalid image file : " + IMG_PATH)

    def _image_del(self):
        self._label.configure(image="")
        self._label.pack()


class TkSTPStateViewer(Tk.Frame):

    def __init__(self, master=None):
        Tk.Frame.__init__(self, master)

        self._init_variable()
        self._setup_frame()

    def _init_variable(self):
        self._node = None
        self._route = None
        self._info = None
        self._topology = None  # h28-n
        self._data_src_func = None
        self.plugin_sel_node_change_func = None

    def _setup_frame(self):
        self.master.title("STP Port State Viewer")

        # init sub-frame
        self._node = NodeList()
        self._route = RouteList()
        self._info = InformationBar()
        self._topology = NetworkTopology()  # h28-n

        self._node.plugin_sel_node_change(self._cb_selected_node_change)

        # add sub-frame
        self._topology.grid(columnspan=2, row=0, column=0,
                            sticky=Tk.EW)    # h28-n
        self._node.grid(row=1, column=0, sticky=Tk.NSEW)
        self._route.grid(row=1, column=1, sticky=Tk.NSEW)
        self._info.grid(columnspan=2, row=2, column=0, sticky=Tk.EW)

        # set resize properties
        self.master.grid_rowconfigure(0, weight=1)
        self.master.grid_columnconfigure(0, weight=1)
        self.master.grid_columnconfigure(1, weight=3)

        # set ttk style
        ttk.Style().map("Treeview",
                        background=[("selected", "LightSkyBlue")],
                        foreground=[])

    def set_data_src_func(self, func):
        self._data_src_func = func

    def set_sel_node_change_func(self, func):
        self.plugin_sel_node_change_func = func

    def init_viewer(self):
        self._node.clear()
        self._route.clear()
        self._info.clear()

    def draw_gui(self, target, node_id, **kwargs):
        if target & NODE:
            self._draw_node_list(node_id, kwargs)

        sel_id = self._node.get_sel_node_id()
        if sel_id is None:
            return

        if sel_id != node_id:
            return

        route_list = self._data_src_func(node_id)  # comp.get_route_data

        if target & ROUTE:
            self._draw_route_list(route_list)

        if target & INFO:
            self._draw_info_bar(route_list)

    def _draw_node_list(self, node_id, kwargs):
        is_missing = None
        try:
            is_missing = kwargs["missing"]
        except KeyError:
            pass

        if is_missing is None:
            # add node
            old_cnt = self._node.get_node_count()

            self._node.add(node_id)

            if not old_cnt:
                self._node.set_sel_node_id(node_id)
        else:
            # update missing node
            self._node.set_missing_node(node_id, is_missing)

    def _draw_route_list(self, route_list):
        if not route_list:
            return

        self._route.show_entire_route(route_list.data)

    def _draw_info_bar(self, route_list):
        if not route_list:
            return

        if route_list.data:
            tm = time.localtime(route_list.tm.sec)
            self._info.set("updated on %04d/%02d/%02d %02d:%02d:%02d",
                           tm.tm_year, tm.tm_mon, tm.tm_mday,
                           tm.tm_hour, tm.tm_min, tm.tm_sec)
        else:
            info_str = ("Communication route to this node"
                        " might have been disconnected")
            self._info.set(info_str)

    def _cb_selected_node_change(self):
        sel_id = self._node.get_sel_node_id()
        self.draw_gui(ROUTE | INFO, sel_id)
        if self.plugin_sel_node_change_func:
            self.plugin_sel_node_change_func()

    def draw_nwtopology(self):
        self._topology.show_nwtopology()

    def get_node_id(self):
        return self._node.get_sel_node_id()

    def draw_nwtopology_del(self):
        self._topology._image_del()  # h28-n


def main():
    pass


if __name__ == "__main__":
    main()
